"""Help goes here"""
import os
import sys
import json
import yaml
import subprocess
import threading
import time
import datetime

from .ui import Colors, Icons
from .spinner import Spinner
from .repo import Repo


class Repos:

    def __init__(self, root):
        self.root = root
        self.repos = {}


    def isGitRepo(self) -> bool:
        self.repo = Repo(self.root)
        self.repo.load()
        return self.repo.git


    def list(self):
        names = []
        for name in os.listdir(self.root):
            if name[0] == ".":
                continue

            dir = os.path.join(self.root, name)
            if os.path.isdir(dir):
                names.append(name)

        sort = sorted(names)
        for name in sort:
            dir = os.path.join(self.root, name)
            repo = Repo(dir)
            self.repos[name] = repo


    def configs(self):
        self.list()

        for _, repo in self.repos.items():
            repo.configs()
            # print(f"REPO {repo.dir}: {configs}")


    def archived(self):
        dirs   = []
        topDir = f"{self.root}/.archived"
        for name in os.listdir(topDir):
            fullDir = os.path.join(topDir, name)
            if os.path.isdir(fullDir):
                dirs.append(name)

        sort = sorted(dirs)
        print("\n".join(sort))


    def save(self, name):
        print(f"Saving repo {name}... ", end="", flush=True)
        repoDir = f"{self.root}/{name}"
        repo = Repo(repoDir)
        isOn = repo.run("git config repos.save")
        # print(f"isOn: {isOn}")

        if isOn == "true":
            repo.run("git add --all")
            repo.run("git commit --message 'Saving it all'")
            print(f"{Colors.GREEN}Saved{Colors.RESET}")
            return True
        else:
            print(f"{Colors.GRAY}Skipped{Colors.RESET}")


    def push(self, name):
        if not self.save(name):
            print(f"Pushing repo {name}... {Colors.GRAY}Skipped{Colors.RESET}", flush=True)
            return

        print(f"Pushing repo {name}... ", end="", flush=True)
        repoDir = f"{self.root}/{name}"
        repo = Repo(repoDir)
        isOn = repo.run("git config repos.push")

        if isOn == "true":
            repo.run("git push origin HEAD")
            print(f"{Colors.GREEN}Pushed{Colors.RESET}")
        else:
            print(f"{Colors.GRAY}Skipped{Colors.RESET}")


    def enable(self, name, feature):
        print(f"Enabling {feature} in repo {name}... ", end="", flush=True)
        repoDir = f"{self.root}/{name}"
        repo = Repo(repoDir)
        isOn = repo.run(f"git config repos.{feature}")

        if isOn == "true":
            print(f"{Colors.GRAY}Skipped{Colors.RESET}")
        else:
            repo.run(f"git config repos.{feature} true")
            print(f"{Colors.GREEN}Enabled{Colors.RESET}")


    def archive(self, name):
        now = datetime.datetime.now().strftime("%Y-%m-%d-%H%M%S")
        print(f"Archiving repo {name}... ", end="", flush=True)
        repoDir = f"{self.root}/{name}"
        archivedDir = f"{self.root}/.archived/{name}@{now}"

        if os.path.isdir(archivedDir):
            print(f" {Colors.GRAY}Skipped{Colors.RESET}")
            return

        if not os.path.isdir(repoDir):
            print(f" {Colors.RED}Fail{Colors.RESET}")
            exit(f"Error: Repo {name} is not an active repo.")

        os.system(f"mkdir -p {self.root}/_archived")
        os.system(f"mv {repoDir} {archivedDir}")
        print(f" {Colors.YELLOW}Archived{Colors.RESET} into {archivedDir}")


    def restore(self, name):
        print(f"Restoring repo {name}... ", end="", flush=True)
        repoDir = f"{self.root}/{name}"
        archDir = f"{self.root}/.archived/{name}"

        if os.path.isdir(repoDir):
            print(f" {Colors.GRAY}Skipped{Colors.RESET}")
            return

        if not os.path.isdir(archDir):
            print(f" {Colors.RED}Fail{Colors.RESET}")
            exit(f"Error: Repo {name} is not an archived repo.")

        os.system(f"mv {self.root}/_archived/{name} {self.root}/{name}")
        print(f" {Colors.GREEN}Restored{Colors.RESET}")


    def flip(self, name):
        print(f"Flipping repo {name}... ", end="", flush=True)
        repoDir = f"{self.root}/{name}"
        archDir = f"{self.root}/.archived/{name}"
        if os.path.isdir(repoDir):
            os.system(f"mkdir -p {self.root}/.archived")
            os.system(f"mv {self.root}/{name} {self.root}/.archived/{name}")
            print(f" {Colors.YELLOW}Archived{Colors.RESET}")
            return

        if os.path.isdir(archDir):
            os.system(f"mv {self.root}/_archived/{name} {self.root}/{name}")
            print(f" {Colors.GREEN}Restored{Colors.RESET}")
            return

        print(f" {Colors.RED}Fail{Colors.RESET}")
        exit(f"Error: Repo {name} is neither an active nor an archived repo.")


    def calc_pads(self, repos):
        self.pads = {
            "changes"  : 0,
            "ahead"    : 0,
            "behind"   : 0,
            "name"     : 0,
            "branch"   : 0,
            "branches" : 0,
            "remotes"  : 0,
        }

        for _, repo in self.repos.items():
            # repo.ago = self.ago(repo.modified)
            # if len(repo.ago) > pads["ago"]:
            #     pads["ago"] = len(repo.ago)
            if len(repo.name) > self.pads["name"]:
                self.pads["name"] = len(repo.name)

            if not repo.git:
                repo.name = repo.name + "/"
                continue

            if len(repo.branch) > self.pads["branch"]:
                self.pads["branch"] = len(repo.branch)
            if len(str(len(repo.branches))) > self.pads["branches"]:
                self.pads["branches"] = len(str(len(repo.branches)))
            if len(str(len(repo.remotes))) > self.pads["remotes"]:
                self.pads["remotes"] = len(str(len(repo.remotes)))
            if len(str(repo.changes)) > self.pads["changes"]:
                self.pads["changes"] = len(str(repo.changes))
            if len(str(repo.ahead)) > self.pads["ahead"]:
                self.pads["ahead"] = len(str(repo.ahead))
            if len(str(repo.behind)) > self.pads["behind"]:
                self.pads["behind"] = len(str(repo.behind))


    def text(self):
        print(f"{Colors.GRAY}  Repos in {Colors.GREEN}{self.root}{Colors.RESET}\n")

        self.calc_pads(self.repos)
        self.total = {
            "dir":      0,
            "solo":     0,
            "detached": 0,
            "clean":    0,
            "changed":  0,
            "ahead":    0,
            "behind":   0,
        }

        status_pad = 8 + self.pads['changes'] + self.pads['ahead'] + self.pads['behind'] + self.pads['branches'] + self.pads['remotes']
        # if REPOS_REMOTES:
        status_pad += 2 + self.pads['remotes']

        print(f"  {Colors.GRAY}{self.pad('STATUS', status_pad, False)}    {self.pad('NAME', self.pads['name'], False)}    {self.pad('BRANCH', self.pads['branch'], False)}{Colors.RESET}")
        print(f"  {Colors.GRAY}{'─' * status_pad}    {'─' * (self.pads['name'])}    {'─' * (self.pads['branch'] + 2)}{Colors.RESET}")

        for _, repo in self.repos.items():
            changes = f"{Colors.GRAY}{Icons.DOT}{Colors.RESET}"
            ahead   = f"{Colors.GRAY}{Icons.DOT}{Colors.RESET}"
            behind  = f"{Colors.GRAY}{Icons.DOT}{Colors.RESET}"

            if repo.changes > 0:
                changes = f"{Colors.YELLOW}{repo.changes}{Icons.DIFF}{Colors.RESET}"

            if repo.ahead > 0:
                ahead = f"{Colors.GREEN}{repo.ahead}{Icons.UP}{Colors.RESET}"

            if repo.behind > 0:
                behind = f"{Colors.PURPLE}{repo.behind}{Icons.DOWN}{Colors.RESET}"

            if not repo.git:
                self.total["dir"] += 1
                color   = Colors.BLUE
                changes = ""
                ahead   = ""
                behind  = ""

            elif len(repo.remotes) < 1:
                self.total["solo"] += 1
                color = Colors.RED
                ahead = ""
                behind = f"{Colors.RED}{Icons.FLAG}{Colors.RESET}"

            elif repo.upstream is None:
                self.total["detached"] += 1
                color = Colors.ORANGE
                ahead = f"{Colors.ORANGE}{Icons.FLAG}{Colors.RESET}"
                behind = ""

            if repo.changes > 0:
                self.total["changed"] += 1
                color = Colors.YELLOW

            elif repo.behind > 0:
                self.total["behind"] += 1
                color = Colors.PURPLE

            elif repo.ahead > 0:
                self.total["ahead"] += 1
                color = Colors.GREEN

            elif repo.changes + repo.ahead + repo.behind == 0:
                if repo.git:
                    self.total["clean"] += 1
                    if repo.upstream:
                        color = Colors.GRAY

            else:
                print(repo)
                raise Exception(f"What happened here?")

            if len(repo.branches) == 1:
                branches = f"{Colors.GRAY}{Icons.DOT}{Colors.RESET}"
            elif len(repo.branches) > 1:
                branches = f"{Colors.GRAY}{str(len(repo.branches))}{Colors.RESET}"
            else:
                branches = ""

            if len(repo.remotes) == 1:
                remotes = f"{Colors.GRAY}{Icons.DOT}{Colors.RESET}"
            elif len(repo.remotes) > 1:
                remotes = f"{Colors.GRAY}{str(len(repo.remotes))}{Colors.RESET}"
            else:
                remotes = ""

            status = self.status(changes, ahead, behind, branches, remotes)
            text = f"{color}  "
            text += f"{self.pad(repo.name, self.pads['name'], False)}    "
            # text += f"{self.pad(repo.branch, self.pads['branch'], False)}"

            text += f"{repo.icon} {repo.branch}"
            text += f"{Colors.RESET}"
            print(f" {status}  {text}")

        self.stats()


    def status(self, changes: str, ahead: str, behind: str, branches: str, remotes: str) -> str:
        text = f"{self.pad(changes, self.pads['changes'] + 2)} "
        text += f"{self.pad(behind, self.pads['behind'] + 2)} "
        text += f"{self.pad(ahead, self.pads['ahead'] + 2)}"
        # if REPOS_REMOTES:
        text += f"{self.pad(remotes, self.pads['remotes'] + 2)}"
        text += f"{self.pad(branches, self.pads['branches'] + 2)}"

        return f"{text}"

    # def test(self):
    #     pad = 6
    #     tests = {
    #         "\033[38;5;220mBlah\033[0m": "--\033[38;5;220mBlah\033[0m",
    #         "\033[1mBla\033[0m": "---\033[1mBla\033[0m",
    #         "B": "-----B",
    #     }
    #     for text, expected in tests.items():
    #         returned = self.pad(text, pad)
    #         if returned != expected:
    #             print(f"Failed test on '{text}': expected '{expected}' vs returned '{returned}'.")
    #             exit(1)

    #     exit()


    def pad(self, text: str, pad: int, right: bool = True) -> str:
        # print(f"==> {len(text)}")
        # 1. '\033[38;5;220mBlah\033[0m'
        # 2. '\033[1mBlah\033[0m'
        # 3. 'Blah'
        fill = " "
        tag = "\033["
        if tag in text:
            pos = text.find(tag)
            end = text.find("m", pos)
            plain = text[end+1:]
            plain = plain.replace(f"{tag}0m", "")
            # print(f"==> PLAIN_TEXT: {plain}")
            pad = pad + len(text) - len(plain)
            # print(f"==> NEW PAD: {pad}")

        if right:
            return text.rjust(pad, fill)
        return text.ljust(pad, fill)


    # def ago(self, value):
    #     # print(f"AGO {value}")
    #     # if value is None:
    #     #     return ""

    #     timestamp = time.time()
    #     duration = math.floor(timestamp - value)

    #     SECOND = 1
    #     MINUTE = SECOND * 60
    #     HOUR   = MINUTE * 60
    #     DAY    = HOUR   * 24
    #     MONTH  = DAY    * 30
    #     YEAR   = DAY    * 365
    #     # WEEK   = DAY    * 7

    #     if duration < SECOND:
    #         return "A moment ago"

    #     if duration < MINUTE:
    #         return f"{math.floor(duration / SECOND)}s"

    #     if duration < HOUR:
    #         return f"{math.floor(duration / MINUTE)}m"

    #     if duration < DAY:
    #         return f"{math.floor(duration / HOUR)}h"

    #     if duration < MONTH:
    #         return f"{math.floor(duration / DAY)}d"

    #     if duration < YEAR:
    #         return f"{math.floor(duration / MONTH)}m"

    #     diff = datetime.datetime.fromtimestamp(value)
    #     return time.strftime("%Y-%m-%d", diff.timetuple())


    def stats(self):
        report = ""
        if self.total["dir"]:
            report += f"\n{self.total['dir']:9} directories"

        if self.total["solo"]:
            report += f"\n{self.total['solo']:9} without a remote {Colors.RED}{Icons.FLAG}{Colors.PALE}"

        if self.total["detached"]:
            # report += f"\n{self.total['detached']:9} without upstream {Colors.ORANGE}⚑{Colors.PALE}"
            report += f"\n{self.total['detached']:9} without upstream {Colors.ORANGE}{Icons.FLAG}{Colors.PALE}"

        if self.total["changed"]:
            report += f"\n{self.total['changed']:9} changed"

        if self.total["behind"]:
            report += f"\n{self.total['behind']:9} behind"

        if self.total["ahead"]:
            report += f"\n{self.total['ahead']:9} ahead"

        if self.total["clean"]:
            report += f"\n{self.total['clean']:9} clean"

        print(f"{Colors.PALE}{report}{Colors.RESET}")


    # @yaspin(text=f"Loading from root...", color="green")
    # @spinner
    def load(self):
        # if self.loaded:
        #     return

        self.list()
        threads = []

        showSpinner = bool(os.environ.get("REPOS_SPINNER", "false"))
        if showSpinner:
            spinner = threading.Thread(
                target=Spinner.start,
                args=(f" Loading from \033[32;1m{self.root}\033[0m...",)
            )
            spinner.start()

        for _, repo in self.repos.items():
            thread = threading.Thread(target=repo.load)
            threads.append(thread)
            thread.start()

        for thread in threads:
            thread.join()

        if showSpinner:
            Spinner.stop()
        Spinner.show()
        self.loaded = True

    def show(self, name: str):
        repo = self.repos[name]
        print(repo)


    def export(self, format: str):
        self.load()
        data = {
            "repos": {}
        }
        for dir, repo in self.repos.items():
            data["repos"][dir] = repo.dict()

        if format == "json":
            print(json.dumps(data, indent=2))
            return True

        if format == "yaml":
            print(yaml.dump(data, indent=2))
            return True



import os
import time


class Spinner:
    CHARS = ['⠏', '⠛', '⠹', '⠼', '⠶', '⠧']

    active = None
    index = 0

    @classmethod
    def start(cls, text):
        cls.active = True
        os.system("tput civis")
        while True:
            for i in range(len(cls.CHARS)):
                if not cls.active:
                    return

                cls.index += 1
                if cls.index > 1:
                    os.system("tput cuu1")
                print(f"\r\033[32;1m{cls.CHARS[i]}\033[0m {text}", end=None)
                time.sleep(0.1)

    @classmethod
    def stop(cls):
        cls.active = False
        os.system("tput cuu1")
        os.system("tput el")
        cls.show()

    @classmethod
    def show(cls):
        os.system("tput cvvis")

    # @classmethod
    # def __call__(cls, fn):
    #     print(f"__CALL__")
    #     return fn

    # # @classmethod
    # def __enter__(cls):
    #     print(f"__ENTER__")

    # # @classmethod
    # def __exit__(cls):
    #     print(f"__EXIT__")



import os
import yaml
import subprocess


class Repo:

    def __init__(self, dir):
        self.dir      = dir
        self.name     = os.path.basename(dir)
        self.git      = False

        self.branch   = ""
        self.ref      = ""
        self.changes  = 0
        self.ahead    = 0
        self.behind   = 0
        self.branches = []
        self.remotes  = []

        self.upstream = None
        self.remote   = None
        self.url      = None
        self.config   = None
        self.icon     = " "
        # self.modified = None


    def load(self):
        # stat = os.stat(self.dir)
        # print(f"STAT {self.name} {stat.st_mtime}")
        # self.modified = stat.st_mtime

        self.git = bool(self.run("git rev-parse --is-inside-work-tree"))
        if self.git is False:
            return

        self.ref      = self.run("git symbolic-ref HEAD")
        self.branch   = self.ref.replace("refs/heads/", "")
        self.changes  = int(self.run("git status --porcelain | wc -l"))
        # self.remotes  = int(self.run("git remote | wc -l"))

        self.loadConfig()

        branches = self.run("git branch | cut -c 3-")
        if branches != "":
            self.branches = branches.split("\n")

        remotes = self.run("git remote")
        if remotes == "":
            return

        self.remotes = remotes.split("\n")
        if len(self.remotes) < 1:
            return

        upstream = self.run("git rev-parse --abbrev-ref @{u}")
        remote   = self.run(f"git config branch.{self.branch}.remote")

        self.upstream = upstream if upstream else None
        self.remote   = remote if remote else None

        if self.remote:
            self.url = self.run(f"git config remote.{self.remote}.url")
            if "git@github.com:" in self.url:
                self.icon = ""
            elif "https://github.com/" in self.url:
                self.icon = ""
            elif "@bitbucket.org/" in self.url:
                self.icon = ""
            elif "git@gitlab.com:" in self.url:
                self.icon = ""
            elif "git.heroku.com/" in self.url:
                self.icon = "\ue77b"
            elif "://git-codecommit." in self.url:
                self.icon = "\uf270"

        if self.upstream:
            cmd = f"git rev-list --count --left-right {self.upstream}...HEAD"
            diff = self.run(cmd).split("\t")
            self.behind = int(diff[0])
            self.ahead = int(diff[1])


    def loadConfig(self):
        config = os.path.join(self.dir, ".git", "repos.yaml")
        exists = os.path.isfile(config)
        if not exists:
            return

        with open(config, "r") as f:
            text = f.read()

        if len(text.strip()) < 1:
            return

        data = yaml.safe_load(text)
        self.config = data


    def saveConfig(self):
        config = os.path.join(self.dir, ".git", "repos.yaml")

        with open(config, "w") as f:
            yaml.dump(self.config, f)


    # git remote add origin git@github.com:jpedro/danger.git
    def __str__(self):
        return f"""
        Repo
            name      {self.name}
            dir       {self.dir}
            git       {self.git}

            branch    {self.branch}
            ref       {self.ref}
            changes   {self.changes}
            behind    {self.behind}
            ahead     {self.ahead}
            branches  {self.branches}
            remotes   {self.remotes}

            remote    {self.remote}
            url       {self.url}
            upstream  {self.upstream}
            config    {self.config}
        """

    def dict(self):
        return {
            "name":      self.name,
            "dir":       self.dir,
            "git":       self.git,

            "branch":    self.branch,
            "ref":       self.ref,
            "changes":   self.changes,
            "behind":    self.behind,
            "ahead":     self.ahead,
            "branches":  self.branches,
            "remotes":   self.remotes,

            "remote":    self.remote,
            "url":       self.url,
            "upstream":  self.upstream,
            "config":    self.config,
        }

    def configs(self) -> dict:
        if self.config:
            return self.config

        self.config = {}
        # print(f"Loading {self.dir} config")
        lines = self.run(f"git config --list").split("\n")
        for line in lines:
            pos = line.find("=")
            if pos > -1:
                parts = line.split("=")
                key = parts[0]
                if key.find("repos.") == 0:
                    key = key[len("repos:"):]
                    val = "=".join(parts[1:])
                    # print(f"Found {key}: {val}")
                    self.config[key] = val

        return self.config

    def run(self, cmd) -> str:
        cmd = f"cd {self.dir} && {cmd}"
        try:
            res = subprocess.check_output(cmd, shell=True, stderr=subprocess.DEVNULL)
            dec = res.decode("UTF-8").strip()
            # if dec == "":
            #     return None
            return dec

        except Exception as e:
            return None



#!/usr/bin/env python3
"""NAME
    repos —  Manages your git repos

USAGE
    repos                       # Lists all repos in text format
    repos export --json         # Exports all repos as json
    repos export --yaml         # Exports all repos as yaml
    repos show REPO             # Shows the repo details
    repos save                  # Commits local changes
    repos push                  # Pushes up to the upstream
    repos pull                  # Pulls from the upstream
    repos sync                  # Pull from the upstream and pushes up
    repos help                  # Shows this help
    repos version               # Prints the current version
"""
import os
import sys
import time

from .ui import Colors
from .repos import Repos


REPOS_TIMER = bool(os.environ.get("REPOS_TIMER", ""))

class CliException(BaseException):
    pass


class Cli:

    def __init__(self, repos: Repos):
        self.repos = repos


    def exportCmd(self, *args):
        format = args[1] if len(args) > 1 else "--yaml"
        if format == "--json":
            self.repos.export("json")
            return

        if format == "--yaml":
            self.repos.export("yaml")
            return

        raise CliException(f"Unknown export format {format}.")


    def slowCmd(self, *_):
        self.repos.list()
        for _, repo in self.repos.items():
            repo.load()
        self.textCmd()


    def textCmd(self, *_):
        self.repos.load()
        self.repos.text()


    def helpCmd(self, *args):
        print(__doc__)
        exit()


    def versionCmd(self, *_):
        print(VERSION)
        exit()


    # def saveCmd(self, *args):
    #     repos = args[1:]
    #     if len(self.repos.repos) > 0:
    #         for repo in repos:
    #             self.save(repo)
    #     else:
    #         print(f"Saving all configured repos")
    #         self.configs()
    #         for _, repo in self.repos.items():
    #             config = repo.config
    #             save = config.get("save", False)
    #             if save == "true":
    #                 # print(f"REPO {repo.dir}: {repo.config}")
    #                 self.save(repo.name)


    # def enableCmd(self, *args):
    #     name = args[1]
    #     for arg in args[2:]:
    #         self.enable(name, arg)


    # def pushCmd(self, *args):
    #     for arg in args[1:]:
    #         self.push(arg)
    #     # self._load()
    #     # for dir, repo in self.repos.items():
    #     #     if repo.changes > 0:
    #     #         print(f"{repo.changes:2}  {dir}")
    #     #         repo.run("git add --all")
    #     #         repo.run("git commit --message 'Saving it all'")


    def archiveCmd(self, *args):
        for arg in args[1:]:
            self.repos.archive(arg)


    def archivedCmd(self, *_):
        self.repos.archived()


    def restoreCmd(self, *args):
        for arg in args[1:]:
            self.repos.restore(arg)


    def flipCmd(self, *args):
        for arg in args[1:]:
            self.repos.flip(arg)


    def showCmd(self, *args):
        self.repos.load()
        for arg in args[1:]:
            self.repos.show(arg)


    def todosCmd(self, *args):
        print()
        print("TODOS:")
        print("- Sync colors with prompt")
        print("- Implement repo config:")
        print()
        print("    repos config repo1                  # Shows all configs for repo1")
        print("    repos config repo1,repo2 key1,key2  # Shows key1 and key2 configs")
        print("    repos config repo1,repo2 key=value  # Sets value for key")
        print()
        print("    Available configs:")
        print("       repos.save      always| never | ask")
        print("       repos.push      always| never | ask")
        print("       repos.fetch     always| never | ask")
        print("       repos.sync      always| never | ask")


    # def configCmd(self, *args):
    #     name  = args[1] if len(args) > 1 else None
    #     key   = args[2] if len(args) > 2 else None
    #     value = args[3] if len(args) > 3 else None

    #     if name is None:
    #          print("Usage: repos config <repo>")
    #          exit(1)

    #     path = os.path.join(self.root, name)
    #     repo = Repo(path)
    #     # configFile = os.path.join(path, ".git", "repos.yaml")
    #     # configExists = os.path.isfile(configFile)
    #     # print(f"Running config repo {name}: {key} --> {value}")

    #     repo.load()

    #     if not repo.git:
    #         print(f"Error: Repo {name} is not a valid git repo")
    #         exit(1)

    #     if key is None:
    #         print(yaml.dump(repo.config))
    #         return

    #     if value is None:
    #         print(repo.config(key, "(none)"))
    #         return

    #     data = {}
    #     if configExists:
    #         with open(configFile, "r") as f:
    #             text = f.read()
    #         if len(text.strip()) > 0:
    #             data = yaml.safe_load(text)

    #     data[key] = value
    #     with open(configFile, "w") as f:
    #         yaml.dump(
    #             data,
    #             f,
    #             sort_keys=False,
    #             indent=2,
    #         )

    #     # print(f"Saved config for repo {name}: {key} = {value} in {configFile} file.")
    #     print(f"Saved config for repo '{name}': '{key}' = '{value}'")


def main():
    if len(sys.argv) < 2:
        cmd = "text"
        args = []
    else:
        cmd = sys.argv[1]
        args = sys.argv[1:]

    # root  = os.path.expanduser("~/code")
    root = os.getcwd()
    repos = Repos(root)
    if repos.isGitRepo():
        print(f"\033[31;1mError:\033[0m You are running inside a git repo ({root}).")
        exit(1)

    cli = Cli(repos)
    try:
        start = time.perf_counter()
        call = getattr(cli, f"{cmd}Cmd")
        call(cli, *args[1:])

        if REPOS_TIMER:
            done = time.perf_counter() - start
            print(f"\n  {Colors.GRAY}Took {(done * 1000):0.0f} ms{Colors.RESET}", file=sys.stderr)

    except Exception as e:
        print(f"Error: `{e}`.", file=sys.stderr)
        raise


if __name__ == "__main__":
    main()



from setuptools import setup
from repos import cli, VERSION

with open("README.md", "r") as f:
    readme = f.read()

setup(
    name="repos",
    version=VERSION,
    description="Manages git repos inside a directory",
    long_description=readme,
    long_description_content_type="text/markdown",
    readme="README.md",
    author="jpedro",
    author_email="jpedro.barbosa@gmail.com",
    url="https://github.com/jpedro/repos",
    download_url="https://github.com/jpedro/repos/tarball/master",
    keywords="git repos",
    license="MIT",
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "Topic :: Software Development :: Build Tools",
        "License :: OSI Approved :: MIT License",
    ],
    packages=[
        "repos",
    ],
    install_requires=[
        "pyyaml",
        "click",
    ],
    entry_points={
        "console_scripts": [
            "repos=repos.cli:main",
        ],
    },
)
